use std::{env::{self, VarError}, path::PathBuf, str::FromStr, fmt::Debug, time::Duration};

pub struct Config {
    pub shows_file: PathBuf,
    pub global_tags: Vec<String>,
    pub post_interval: Duration,
    pub cohost_email: String,
    pub cohost_password: String,
    pub cohost_page: String,
}

const VAR_PREFIX: &'static str = "SCREENCAP_BOT_";

fn get_var(name: &str) -> Result<String, VarError> {
    env::var(VAR_PREFIX.to_string() + name)
}

fn parse_var<E: Debug, T: FromStr<Err = E>>(name: &str) -> Result<T, VarError> {
    get_var(name).map(|s| s.parse().expect(&format!("Failed to parse {}{}", VAR_PREFIX, name)))
}

fn expect_var(name: &str) -> String {
    get_var(name).expect(&format!("{}{} must be set", VAR_PREFIX, name))
}

pub fn load() -> Config {
    Config {
        shows_file: parse_var("SHOWS_FILE").unwrap_or(PathBuf::from("./shows.yaml")),
        global_tags: get_var("GLOBAL_TAGS").map(|s| s.split(",").map(String::from).collect()).unwrap_or_default(),
        post_interval: Duration::from_secs(parse_var("POST_INTERVAL").unwrap_or(6*3600)),
        cohost_email: expect_var("COHOST_EMAIL"),
        cohost_password: expect_var("COHOST_PASSWORD"),
        cohost_page: expect_var("COHOST_PAGE"),
    }
}
